package weltkante.scanner

abstract class SToken {
  var fragment: String = null
}

abstract class TTerm extends SToken

abstract class TConst extends TTerm

abstract class TNumber extends TConst

abstract class TOpen extends SToken

abstract class TClose extends SToken

final class TNewline extends SToken

final class TInteger extends TNumber

final class TFloat extends TNumber

final class TString extends TConst

final class TStringHead extends TTerm

final class TStringPart extends TTerm

final class TStringTail extends TTerm

final class TSymbol extends SToken

final class TIdent extends SToken

final class TData extends TTerm {
  var content: Array[Byte] = null;
}

final class TOpenGroup extends TOpen

final class TOpenIndex extends TOpen

final class TOpenContext extends TOpen

final class TOpenHint extends TOpen

final class TCloseGroup extends TClose

final class TCloseIndex extends TClose

final class TCloseContext extends TClose

final class TCloseHint extends TClose

final class TSpace extends SToken

final class TTab extends SToken

final class TEnd extends SToken

final class TComment extends SToken

class STokenFactory[TToken <: SToken](val create: () => TToken)

object STokenFactories {
  private def define[TToken <: SToken](create: => TToken) =
    new STokenFactory[TToken](() => create)

  implicit val TNewlineFactory = define(new TNewline)
  implicit val TIntegerFactory = define(new TInteger)
  implicit val TFloatFactory = define(new TFloat)
  implicit val TStringFactory = define(new TString)
  implicit val TStringHeadFactory = define(new TStringHead)
  implicit val TStringPartFactory = define(new TStringPart)
  implicit val TStringTailFactory = define(new TStringTail)
  implicit val TSymbolFactory = define(new TSymbol)
  implicit val TIdentFactory = define(new TIdent)
  implicit val TDataFactory = define(new TData)
  implicit val TOpenGroupFactory = define(new TOpenGroup)
  implicit val TOpenIndexFactory = define(new TOpenIndex)
  implicit val TOpenContextFactory = define(new TOpenContext)
  implicit val TOpenHintFactory = define(new TOpenHint)
  implicit val TCloseGroupFactory = define(new TCloseGroup)
  implicit val TCloseIndexFactory = define(new TCloseIndex)
  implicit val TCloseContextFactory = define(new TCloseContext)
  implicit val TCloseHintFactory = define(new TCloseHint)
  implicit val TSpaceFactory = define(new TSpace)
  implicit val TTabFactory = define(new TTab)
  implicit val TEndFactory = define(new TEnd)
  implicit val TCommentFactory = define(new TComment)
}
